const ExtensionUtils = imports.misc.extensionUtils;
const { St, Clutter, GObject, Gio } = imports.gi;
const { main: Main, popupMenu: PopupMenu, panelMenu: PanelMenu } = imports.ui;

const currentExtension = ExtensionUtils.getCurrentExtension();
const settings = ExtensionUtils.getSettings(
  "org.gnome.shell.extensions.icon-hider-updated"
);

var IconHiderExtension;

class IconIndicator extends PanelMenu.Button {
  _init() {
    super._init(0.0, `${currentExtension.metadata.name} Indicator`, false);
    this._buildUI();
  }

  _buildUI() {
    const box = new St.BoxLayout({ style_class: "panel-button" });
    const icon = new St.Icon({
      icon_name: "view-grid-symbolic",
      icon_size: 20,
    });

    box.add_child(icon);
    this.add_child(box);
  }
}
GObject.registerClass(IconIndicator);

class IconHiderExtensionClass {
  constructor() {
    this.statusArea = Main.panel.statusArea;
    this.hiddenIcons = new Set(settings.get_strv("hidden-icons"));
    this.knownIcons = new Set(settings.get_strv("known-icons"));
    this.indicator = null;
    this.connectSettings();
  }

  connectSettings() {
    settings.connect(
      "changed::hide-indicator-icon",
      this._updateIndicatorVisibility.bind(this)
    );
    settings.connect("changed::hidden-icons", this._syncHiddenIcons.bind(this));
  }

  _toggleIconVisibility(iconRole) {
    if (this.hiddenIcons.has(iconRole)) {
      this.hiddenIcons.delete(iconRole);
      this.statusArea[iconRole].show();
    } else {
      this.hiddenIcons.add(iconRole);
      this.statusArea[iconRole].hide();
    }
    settings.set_strv("hidden-icons", Array.from(this.hiddenIcons));
  }

  _createIndicator() {
    if (!this.indicator) {
      this.indicator = new IconIndicator();
      Main.panel.addToStatusArea(
        "iconHiderUpdated",
        this.indicator,
        1,
        "right"
      );
    }
    this._setupMenu();
  }

  _setupMenu() {
    this.indicator.menu.removeAll();
    this.knownIcons.forEach((iconRole) => {
      const menuItem = new PopupMenu.PopupSwitchMenuItem(
        iconRole,
        !this.hiddenIcons.has(iconRole)
      );
      menuItem.connect("toggled", () => this._toggleIconVisibility(iconRole));
      this.indicator.menu.addMenuItem(menuItem);
    });

    this.indicator.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
    this._addSettingsMenuItem();
  }

  _addSettingsMenuItem() {
    const openSettingsItem = new PopupMenu.PopupMenuItem("Open Settings");
    openSettingsItem.connect("activate", () => ExtensionUtils.openPrefs());
    this.indicator.menu.addMenuItem(openSettingsItem);
  }

  _gatherStatusAreaElements() {
    Object.keys(this.statusArea).forEach((role) => {
      if (!this.statusArea[role].is_finalized && role !== "iconHiderUpdated") {
        this.knownIcons.add(role);
      }
    });
    settings.set_strv("known-icons", Array.from(this.knownIcons));
  }

  _syncHiddenIcons() {
    this.hiddenIcons = new Set(settings.get_strv("hidden-icons"));
    this._applyHiddenIcons();
  }

  _applyHiddenIcons() {
    this.knownIcons.forEach((iconRole) => {
      if (this.statusArea[iconRole]) {
        this.hiddenIcons.has(iconRole)
          ? this.statusArea[iconRole].hide()
          : this.statusArea[iconRole].show();
      }
    });
  }

  _updateIndicatorVisibility() {
    if (this.indicator) {
      settings.get_boolean("hide-indicator-icon")
        ? this.indicator.hide()
        : this.indicator.show();
    }
  }

  enable() {
    setTimeout(() => {
      this._gatherStatusAreaElements();
      this._applyHiddenIcons();
      this._createIndicator();
      this._updateIndicatorVisibility();
    }, 500);
  }

  disable() {
    settings.disconnect("changed::hide-indicator-icon");
    settings.disconnect("changed::hidden-icons");
    if (this.indicator) {
      this.indicator.destroy();
      this.indicator = null;
    }
  }
}

function init() {
  IconHiderExtension = new IconHiderExtensionClass();
  return IconHiderExtension;
}
